
package org.mule.modules.quickbooks.windows.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.quickbooks.windows.processors.StatusMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class StatusDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public StatusDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(StatusMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("realmId")!= null)&&(!StringUtils.isBlank(element.getAttribute("realmId")))) {
            builder.addPropertyValue("realmId", element.getAttribute("realmId"));
        }
        if ((element.getAttribute("appKey")!= null)&&(!StringUtils.isBlank(element.getAttribute("appKey")))) {
            builder.addPropertyValue("appKey", element.getAttribute("appKey"));
        }
        if ((element.getAttribute("realmIdPseudonym")!= null)&&(!StringUtils.isBlank(element.getAttribute("realmIdPseudonym")))) {
            builder.addPropertyValue("realmIdPseudonym", element.getAttribute("realmIdPseudonym"));
        }
        if ((element.getAttribute("authIdPseudonym")!= null)&&(!StringUtils.isBlank(element.getAttribute("authIdPseudonym")))) {
            builder.addPropertyValue("authIdPseudonym", element.getAttribute("authIdPseudonym"));
        }
        Element syncStatusRequestListElement = null;
        syncStatusRequestListElement = DomUtils.getChildElementByTagName(element, "sync-status-request");
        List<Element> syncStatusRequestMapChilds = null;
        if (syncStatusRequestListElement!= null) {
            String syncStatusRequestRef = syncStatusRequestListElement.getAttribute("ref");
            if ((syncStatusRequestRef!= null)&&(!StringUtils.isBlank(syncStatusRequestRef))) {
                if ((!syncStatusRequestRef.startsWith(patternInfo.getPrefix()))&&(!syncStatusRequestRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("syncStatusRequest", new RuntimeBeanReference(syncStatusRequestRef));
                } else {
                    builder.addPropertyValue("syncStatusRequest", syncStatusRequestRef);
                }
            } else {
                ManagedMap syncStatusRequest = new ManagedMap();
                syncStatusRequestMapChilds = DomUtils.getChildElementsByTagName(syncStatusRequestListElement, "sync-status-request");
                if (syncStatusRequestMapChilds!= null) {
                    if (syncStatusRequestMapChilds.size() == 0) {
                        syncStatusRequestMapChilds = DomUtils.getChildElements(syncStatusRequestListElement);
                    }
                    for (Element syncStatusRequestChild: syncStatusRequestMapChilds) {
                        String syncStatusRequestValueRef = syncStatusRequestChild.getAttribute("value-ref");
                        String syncStatusRequestKeyRef = syncStatusRequestChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((syncStatusRequestValueRef!= null)&&(!StringUtils.isBlank(syncStatusRequestValueRef))) {
                            if ((!syncStatusRequestValueRef.startsWith(patternInfo.getPrefix()))&&(!syncStatusRequestValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(syncStatusRequestValueRef);
                            } else {
                                valueObject = syncStatusRequestValueRef;
                            }
                        } else {
                            valueObject = syncStatusRequestChild.getTextContent();
                        }
                        if ((syncStatusRequestKeyRef!= null)&&(!StringUtils.isBlank(syncStatusRequestKeyRef))) {
                            keyObject = new RuntimeBeanReference(syncStatusRequestKeyRef);
                        } else {
                            keyObject = syncStatusRequestChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = syncStatusRequestChild.getTagName();
                        }
                        syncStatusRequest.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("syncStatusRequest", syncStatusRequest);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
